@@ -22,7 +22,7 @@ use crate::xdmerror::*;
 use crate::parsecommon::*;
 use crate::evaluate::{
     DynamicContext,
-    evaluate,
+    StaticContext,
     static_analysis,
     Constructor,
     NameTest, WildcardOrName,
@@ -30,7 +30,7 @@ use crate::evaluate::{
     Axis,
     ArithmeticOperator, ArithmeticOperand,
     Function,
-    StaticContext,
+    evaluate,
 };
 
 // Expr ::= ExprSingle (',' ExprSingle)* ;
@@ -1962,178 +1962,17 @@ mod tests {
 	}
     }
 
-    #[test]
-    fn xnode_root() {
-      let dc = DynamicContext::new();
-      let d = roxmltree::Document::parse("<Level1><Level2>one</Level2><Level2>two</Level2><Level2>three</Level2></Level1>").expect("failed to parse XML");
-      let s = vec![Rc::new(Item::XNode(d.root().first_child().unwrap()))];
-      let c = parse("/").expect("unable to parse XPath \"/\"");
-      let e = evaluate(&dc, Some(s), Some(0), &c)
-        .expect("evaluation failed");
-      if e.len() == 1 {
-        assert_eq!(e[0].to_xml(), "<Level1><Level2>one</Level2><Level2>two</Level2><Level2>three</Level2></Level1>");
-      } else {
-        panic!("sequence does not have 1 item: \"{}\"", e.to_string())
-      }
-    }
     #[test]
     fn nomxpath_parse_root_step_1() {
         let _e = parse("/child::a").expect("failed to parse expression \"/child::a\"");
 	assert!(true) // TODO: check the sequence constructor
     }
     #[test]
-    fn xnode_step_1() {
-      let dc = DynamicContext::new();
-      let d = roxmltree::Document::parse("<Level1><Level2>one</Level2><Level2>two</Level2><Level2>three</Level2></Level1>").expect("failed to parse XML");
-      let s = vec![Rc::new(Item::XNode(d.root().first_child().unwrap()))];
-      let c = parse("/child::*").expect("failed to parse expression \"/child::*\"");
-      let e = evaluate(&dc, Some(s), Some(0), &c)
-        .expect("evaluation failed");
-      if e.len() == 1 {
-        assert_eq!(e[0].to_xml(), "<Level1><Level2>one</Level2><Level2>two</Level2><Level2>three</Level2></Level1>");
-      } else {
-        panic!("sequence does not have 1 item, it has {}: \"{}\"", e.len(), e.to_string())
-      }
-    }
-    #[test]
     fn nomxpath_parse_root_step_2() {
         let _e = parse("/child::a/child::b").expect("failed to parse expression \"/child::a/child::b\"");
 	assert!(true) // TODO: check the sequence constructor
     }
     #[test]
-    fn xnode_step_nodetest_pos() {
-      let dc = DynamicContext::new();
-      let d = roxmltree::Document::parse("<Level1/>").expect("failed to parse XML");
-      let s = vec![Rc::new(Item::XNode(d.root().first_child().unwrap()))];
-      let c = parse("/child::Level1").expect("failed to parse expression \"/child::Level1\"");
-      let e = evaluate(&dc, Some(s), Some(0), &c)
-        .expect("evaluation failed");
-      if e.len() == 1 {
-        assert_eq!(e[0].to_xml(), "<Level1/>");
-      } else {
-        panic!("sequence does not have 1 item, it has {}: \"{}\"", e.len(), e.to_string())
-      }
-    }
-    #[test]
-    fn xnode_step_nodetest_neg() {
-      let dc = DynamicContext::new();
-      let d = roxmltree::Document::parse("<Level1/>").expect("failed to parse XML");
-      let s = vec![Rc::new(Item::XNode(d.root().first_child().unwrap()))];
-      let c = parse("/child::Test").expect("failed to parse expression \"/child::Test\"");
-      let e = evaluate(&dc, Some(s), Some(0), &c)
-        .expect("evaluation failed");
-      if e.len() == 0 {
-        assert!(true)
-      } else {
-        panic!("sequence has more than 1 item, it has {}: \"{}\"", e.len(), e.to_string())
-      }
-    }
-    #[test]
-    fn xnode_step_2() {
-      let dc = DynamicContext::new();
-      let d = roxmltree::Document::parse("<Level1><Level2>one</Level2><Level2>two</Level2><Level2>three</Level2></Level1>").expect("failed to parse XML");
-      let s = vec![Rc::new(Item::XNode(d.root().first_child().unwrap()))];
-      let c = parse("/child::*/child::*").expect("failed to parse expression \"/child::a/child::b\"");
-      let e = evaluate(&dc, Some(s), Some(0), &c)
-        .expect("evaluation failed");
-      if e.len() == 3 {
-        assert_eq!(e[0].to_xml(), "<Level2>one</Level2>");
-        assert_eq!(e[1].to_xml(), "<Level2>two</Level2>");
-        assert_eq!(e[2].to_xml(), "<Level2>three</Level2>");
-      } else {
-        panic!("sequence does not have 3 items, it has {}: \"{}\"", e.len(), e.to_string())
-      }
-    }
-    #[test]
-    fn xnode_descendant_1() {
-      let dc = DynamicContext::new();
-      let d = roxmltree::Document::parse("<Test><level1><level2><level3>1 1 1</level3><level3>1 1 2</level3></level2><level2><level3>1 2 1</level3><level3>1 2 2</level3></level2></level1><level1>not me</level1></Test>").expect("failed to parse XML");
-      let s = vec![Rc::new(Item::XNode(d.root().first_child().unwrap().first_child().unwrap()))];
-      let c = parse("descendant::*").expect("failed to parse expression \"descendant::*\"");
-      let e = evaluate(&dc, Some(s), Some(0), &c)
-        .expect("evaluation failed");
-      assert_eq!(e.len(), 6);
-      assert_eq!(e[1].to_xml(), "<level3>1 1 1</level3>")
-    }
-    #[test]
-    fn xnode_descendantorself_1() {
-      let dc = DynamicContext::new();
-      let d = roxmltree::Document::parse("<Test><level1><level2><level3>1 1 1</level3><level3>1 1 2</level3></level2><level2><level3>1 2 1</level3><level3>1 2 2</level3></level2></level1><level1>not me</level1></Test>").expect("failed to parse XML");
-      let s = vec![Rc::new(Item::XNode(d.root().first_child().unwrap().first_child().unwrap()))];
-      let c = parse("descendant-or-self::*").expect("failed to parse expression \"descendant-or-self::*\"");
-      let e = evaluate(&dc, Some(s), Some(0), &c)
-        .expect("evaluation failed");
-      assert_eq!(e.len(), 7);
-      assert_eq!(e[2].to_xml(), "<level3>1 1 1</level3>")
-    }
-    #[test]
-    fn xnode_ancestor_1() {
-      let dc = DynamicContext::new();
-      let d = roxmltree::Document::parse("<Test><level1><level2><level3>1 1 1</level3><level3>1 1 2</level3></level2><level2><level3>1 2 1</level3><level3>1 2 2</level3></level2></level1><level1>not me</level1></Test>").expect("failed to parse XML");
-      let s = vec![Rc::new(Item::XNode(d.root().first_child().unwrap().first_child().unwrap().first_child().unwrap().first_child().unwrap()))];
-      let c = parse("ancestor::*").expect("failed to parse expression \"ancestor::*\"");
-      let e = evaluate(&dc, Some(s), Some(0), &c)
-        .expect("evaluation failed");
-      assert_eq!(e.len(), 3);
-    }
-    #[test]
-    fn xnode_ancestororself_1() {
-      let dc = DynamicContext::new();
-      let d = roxmltree::Document::parse("<Test><level1><level2><level3>1 1 1</level3><level3>1 1 2</level3></level2><level2><level3>1 2 1</level3><level3>1 2 2</level3></level2></level1><level1>not me</level1></Test>").expect("failed to parse XML");
-      let s = vec![Rc::new(Item::XNode(d.root().first_child().unwrap().first_child().unwrap().first_child().unwrap()))];
-      let c = parse("ancestor-or-self::*").expect("failed to parse expression \"ancestor-or-self::*\"");
-      let e = evaluate(&dc, Some(s), Some(0), &c)
-        .expect("evaluation failed");
-      assert_eq!(e.len(), 3);
-    }
-    #[test]
-    fn xnode_followingsibling_1() {
-      let dc = DynamicContext::new();
-      let d = roxmltree::Document::parse("<Test><level1><level2><level3>1 1 1</level3><level3>1 1 2</level3></level2><level2><level3>1 2 1</level3><level3>1 2 2</level3></level2></level1><level1>not me</level1></Test>").expect("failed to parse XML");
-      let s = vec![Rc::new(Item::XNode(d.root().first_child().unwrap().first_child().unwrap().first_child().unwrap().first_child().unwrap()))];
-      let c = parse("following-sibling::*").expect("failed to parse expression \"following-sibling::*\"");
-      let e = evaluate(&dc, Some(s), Some(0), &c)
-        .expect("evaluation failed");
-      assert_eq!(e.len(), 1);
-      assert_eq!(e.to_xml(), "<level3>1 1 2</level3>");
-    }
-    #[test]
-    fn xnode_precedingsibling_1() {
-      let dc = DynamicContext::new();
-      let d = roxmltree::Document::parse("<Test><level1><level2><level3>1 1 1</level3><level3>1 1 2</level3></level2><level2><level3>1 2 1</level3><level3>1 2 2</level3></level2></level1><level1>not me</level1></Test>").expect("failed to parse XML");
-      let s = vec![Rc::new(Item::XNode(d.root().first_child().unwrap().first_child().unwrap().first_child().unwrap().last_child().unwrap()))];
-      let c = parse("preceding-sibling::*").expect("failed to parse expression \"preceding-sibling::*\"");
-      let e = evaluate(&dc, Some(s), Some(0), &c)
-        .expect("evaluation failed");
-      assert_eq!(e.len(), 1);
-      assert_eq!(e.to_xml(), "<level3>1 1 1</level3>");
-    }
-    #[test]
-    fn xnode_following_1() {
-      let dc = DynamicContext::new();
-      let d = roxmltree::Document::parse("<Test><level1><level2><level3>1 1 1</level3><level3>1 1 2</level3></level2><level2><level3>1 2 1</level3><level3>1 2 2</level3></level2></level1><level1>not me</level1></Test>").expect("failed to parse XML");
-      let s = vec![Rc::new(Item::XNode(d.root().first_child().unwrap().first_child().unwrap().first_child().unwrap().first_child().unwrap()))];
-      let c = parse("following::*").expect("failed to parse expression \"following::*\"");
-      let e = evaluate(&dc, Some(s), Some(0), &c)
-        .expect("evaluation failed");
-      assert_eq!(e.len(), 4);
-      assert_eq!(e.to_xml(), "<level2><level3>1 2 1</level3><level3>1 2 2</level3></level2><level3>1 2 1</level3><level3>1 2 2</level3><level1>not me</level1>");
-    }
-    #[test]
-    fn xnode_preceding_1() {
-      let dc = DynamicContext::new();
-      let d = roxmltree::Document::parse("<Test><level1><level2><level3>1 1 1</level3><level3>1 1 2</level3></level2><level2><level3>1 2 1</level3><level3>1 2 2</level3></level2></level1><level1>not me</level1></Test>").expect("failed to parse XML");
-      let s = vec![Rc::new(Item::XNode(d.root().first_child().unwrap().last_child().unwrap()))];
-      let c = parse("preceding::*").expect("failed to parse expression \"preceding::*\"");
-      let e = evaluate(&dc, Some(s), Some(0), &c)
-        .expect("evaluation failed");
-      assert_eq!(e.len(), 7);
-      assert_eq!(e[0].to_name(), "level1");
-      assert_eq!(e[1].to_name(), "level2");
-      assert_eq!(e[2].to_name(), "level3");
-      assert_eq!(e[2].to_xml(), "<level3>1 1 1</level3>");
-    }
-    #[test]
     fn nomxpath_parse_desc_or_self_1() {
         let _e = parse("//child::a").expect("failed to parse expression \"//child::a\"");
 	assert!(true) // TODO: check the sequence constructor
@@ -2179,80 +2018,6 @@ mod tests {
 	assert!(true) // TODO: check the sequence constructor
     }
 
-    #[test]
-    fn parse_eval_predicate_pos() {
-      let dc = DynamicContext::new();
-      let x = roxmltree::Document::parse("<Test><a><b/></a><a><c/></a></Test>").expect("failed to parse XML");
-      let d = vec![Rc::new(Item::XNode(x.root().first_child().unwrap()))];
-      let e = parse("/child::*/child::*[child::b]").expect("failed to parse expression \"//child::*/child::*[child::b]\"");
-      let s = evaluate(&dc, Some(d), Some(0), &e).expect("evaluation failed");
-      assert_eq!(s.len(), 1);
-      assert_eq!(s.to_xml(), "<a><b/></a>")
-    }
-    #[test]
-    fn parse_eval_predicate_neg() {
-      let dc = DynamicContext::new();
-      let x = roxmltree::Document::parse("<Test><a><b/></a><a><c/></a></Test>").expect("failed to parse XML");
-      let d = vec![Rc::new(Item::XNode(x.root().first_child().unwrap()))];
-      let e = parse("/child::*[child::b]").expect("failed to parse expression \"/child::*[child::b]\"");
-      let s = evaluate(&dc, Some(d), Some(0), &e).expect("evaluation failed");
-      assert_eq!(s.len(), 0)
-    }
-    #[test]
-    fn parse_eval_fncall_position() {
-      let dc = DynamicContext::new();
-      let x = roxmltree::Document::parse("<Test><a><b/></a><a><c/></a></Test>").expect("failed to parse XML");
-      let d = vec![Rc::new(Item::XNode(x.root().first_child().unwrap()))];
-      let mut e = parse("/child::*/child::*[position() eq 1]").expect("failed to parse expression \"/child::*/child::*[position() eq 1]\"");
-      let mut sc = StaticContext::new_with_builtins();
-      static_analysis(&mut e, &mut sc);
-      let s = evaluate(&dc, Some(d), Some(0), &e).expect("evaluation failed");
-      assert_eq!(s.to_xml(), "<a><b/></a>")
-    }
-    #[test]
-    fn parse_eval_fncall_last() {
-      let dc = DynamicContext::new();
-      let x = roxmltree::Document::parse("<Test><a><b/></a><a><c/></a><a><d/></a></Test>").expect("failed to parse XML");
-      let d = vec![Rc::new(Item::XNode(x.root().first_child().unwrap()))];
-      let mut e = parse("/child::*/child::*[position() eq last()]").expect("failed to parse expression \"/child::*/child::*[position() eq last()]\"");
-      let mut sc = StaticContext::new_with_builtins();
-      static_analysis(&mut e, &mut sc);
-      let s = evaluate(&dc, Some(d), Some(0), &e).expect("evaluation failed");
-      assert_eq!(s.to_xml(), "<a><d/></a>")
-    }
-    #[test]
-    fn parse_eval_fncall_count() {
-      let dc = DynamicContext::new();
-      let x = roxmltree::Document::parse("<Test><a><b/></a><a><c/></a><a><d/></a></Test>").expect("failed to parse XML");
-      let d = vec![Rc::new(Item::XNode(x.root().first_child().unwrap()))];
-      let mut e = parse("count(/child::*/child::*)").expect("failed to parse expression \"count(/child::*/child::*)\"");
-      let mut sc = StaticContext::new_with_builtins();
-      static_analysis(&mut e, &mut sc);
-      let s = evaluate(&dc, Some(d), Some(0), &e).expect("evaluation failed");
-      assert_eq!(s.to_string(), "3")
-    }
-    #[test]
-    fn parse_eval_fncall_localname() {
-      let dc = DynamicContext::new();
-      let x = roxmltree::Document::parse("<Test><a><b/></a><a><c/></a><a><d/></a></Test>").expect("failed to parse XML");
-      let d = vec![Rc::new(Item::XNode(x.root().first_child().unwrap()))];
-      let mut e = parse("local-name()").expect("failed to parse expression \"local-name()\"");
-      let mut sc = StaticContext::new_with_builtins();
-      static_analysis(&mut e, &mut sc);
-      let s = evaluate(&dc, Some(d), Some(0), &e).expect("evaluation failed");
-      assert_eq!(s.to_string(), "Test")
-    }
-    #[test]
-    fn parse_eval_fncall_name() {
-      let dc = DynamicContext::new();
-      let x = roxmltree::Document::parse("<Test><a><b/></a><a><c/></a><a><d/></a></Test>").expect("failed to parse XML");
-      let d = vec![Rc::new(Item::XNode(x.root().first_child().unwrap()))];
-      let mut e = parse("name()").expect("failed to parse expression \"name()\"");
-      let mut sc = StaticContext::new_with_builtins();
-      static_analysis(&mut e, &mut sc);
-      let s = evaluate(&dc, Some(d), Some(0), &e).expect("evaluation failed");
-      assert_eq!(s.to_string(), "Test")
-    }
     #[test]
     fn parse_eval_fncall_string() {
       let dc = DynamicContext::new();
@@ -2603,60 +2368,5 @@ mod tests {
       assert_eq!(s.len(), 1);
       assert_eq!(s.to_string(), "not one")
     }
-
-    // Kind Tests
-    #[test]
-    fn xnode_kind_element_1() {
-      let dc = DynamicContext::new();
-      let d = roxmltree::Document::parse("<Test><level1>1<level2/>2<level2/>3<level2/>4<level2/>5<level2/>6<level2/>7</level1></Test>").expect("failed to parse XML");
-      let cons = parse("child::element()").expect("failed to parse element kind expression");
-      let e = evaluate(&dc, Some(vec![Rc::new(Item::XNode(d.root().first_child().unwrap().first_child().unwrap()))]), Some(0), &cons)
-        .expect("evaluation failed");
-      assert_eq!(e.len(), 6);
-      assert_eq!(e[0].to_name(), "level2");
-      assert_eq!(e[1].to_name(), "level2");
-      assert_eq!(e[2].to_name(), "level2");
-      assert_eq!(e[3].to_name(), "level2");
-      assert_eq!(e[4].to_name(), "level2");
-      assert_eq!(e[5].to_name(), "level2");
-    }
-    #[test]
-    fn xnode_kind_text_1() {
-      let dc = DynamicContext::new();
-      let d = roxmltree::Document::parse("<Test><level1>1<level2/>2<level2/>3<level2/>4<level2/>5<level2/>6<level2/>7</level1></Test>").expect("failed to parse XML");
-      let cons = parse("child::text()").expect("failed to parse text kind expression");
-      let e = evaluate(&dc, Some(vec![Rc::new(Item::XNode(d.root().first_child().unwrap().first_child().unwrap()))]), Some(0), &cons)
-        .expect("evaluation failed");
-      assert_eq!(e.len(), 7);
-      assert_eq!(e[0].to_string(), "1");
-      assert_eq!(e[1].to_string(), "2");
-      assert_eq!(e[2].to_string(), "3");
-      assert_eq!(e[3].to_string(), "4");
-      assert_eq!(e[4].to_string(), "5");
-      assert_eq!(e[5].to_string(), "6");
-      assert_eq!(e[6].to_string(), "7");
-    }
-    #[test]
-    fn xnode_kind_any_1() {
-      let dc = DynamicContext::new();
-      let d = roxmltree::Document::parse("<Test><level1>1<level2/>2<level2/>3<level2/>4<level2/>5<level2/>6<level2/>7</level1></Test>").expect("failed to parse XML");
-      let cons = parse("child::node()").expect("failed to parse text kind expression");
-      let e = evaluate(&dc, Some(vec![Rc::new(Item::XNode(d.root().first_child().unwrap().first_child().unwrap()))]), Some(0), &cons)
-        .expect("evaluation failed");
-      assert_eq!(e.len(), 13);
-      assert_eq!(e[0].to_string(), "1");
-      assert_eq!(e[1].to_name(), "level2");
-      assert_eq!(e[2].to_string(), "2");
-      assert_eq!(e[3].to_name(), "level2");
-      assert_eq!(e[4].to_string(), "3");
-      assert_eq!(e[5].to_name(), "level2");
-      assert_eq!(e[6].to_string(), "4");
-      assert_eq!(e[7].to_name(), "level2");
-      assert_eq!(e[8].to_string(), "5");
-      assert_eq!(e[9].to_name(), "level2");
-      assert_eq!(e[10].to_string(), "6");
-      assert_eq!(e[11].to_name(), "level2");
-      assert_eq!(e[12].to_string(), "7");
-    }
 }
 
